from .sendfiles import returndata
from .sendfiles import uploadcsvfortraining
from .sendfiles import getpredictions

from .sendfiles import dolistkeys
from .sendfiles import dolistkeyswithkey
from .sendfiles import dodeletewithkey

from .sendfiles import getpicklezip
from .sendfiles import sendpicklezip
from .sendfiles import deploytoprod
from .sendfiles import nlp
from .sendfiles import nlpclassify
from .sendfiles import algoinfo
from .sendfiles import nlpaudiovideo
from .sendfiles import nlpocr
from .sendfiles import nlpgeosentiment
from .sendfiles import optimize

from .sendfiles import hyperpredictions
from .sendfiles import tcp_echo_client
from .sendfiles import retraining
from .sendfiles import genpdf

from .sendfiles import featureselectionjson


from .sendfiles import viperstats

from .sendfiles import viperlisttopics

from .sendfiles import vipersubscribeconsumer

from .sendfiles import viperunsubscribeconsumer

from .sendfiles import viperproducetotopic
from .sendfiles import viperconsumefromtopic
from .sendfiles import viperhpdepredict

from .sendfiles import viperhpdeoptimize

from .sendfiles import viperhpdetraining

from .sendfiles import viperproducetotopicstream

from .sendfiles import vipercreatetrainingdata
    

from .sendfiles import vipercreatetopic
from .sendfiles import viperconsumefromstreamtopic
from .sendfiles import vipercreatejointopicstreams
    
from .sendfiles import vipercreateconsumergroup
from .sendfiles import viperconsumergroupconsumefromtopic
    
from .sendfiles import vipermodifyconsumerdetails
    
from .sendfiles import vipermodifytopicdetails
    
from .sendfiles import viperactivatetopic
    
from .sendfiles import viperdeactivatetopic
    
from .sendfiles import vipergroupactivate
    
from .sendfiles import vipergroupdeactivate

from .sendfiles import viperanomalytrain
from .sendfiles import viperanomalypredict

from .callmas import dotraining
from .callmas import dopredictions

from .balancedata import balancebigdata

name = "maads"
